//---------------------------------------------------------------------------
// GlyphConvertCommand.cpp
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <UtilCls.h>
#include "GlyphConvertCommand.h"
//---------------------------------------------------------------------------
namespace Moochos { namespace FullGlyph { namespace GlyphConvertCommand {
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
void __fastcall TGlyphConvertCommand::LoadGlyph(Graphics::TBitmap* AGlyph)
{
	try {
		AGlyph->LoadFromFile(FFile);
	}
	catch(EStreamError& e) {
		throw EGlyphConvertError("Cannot load glyph from file: " + FFile);
	}
	catch(EInvalidGraphic& e) {
		throw EGlyphConvertError("Cannot load glyph from file: " + FFile);
	}
}
//---------------------------------------------------------------------------
void __fastcall TGlyphConvertCommand::EditGlyph(Graphics::TBitmap* AGlyph)
{
	if(AGlyph->Width == 0 || AGlyph->Height == 0) {
		return;
	}
	TPtr<Graphics::TBitmap> image = new Graphics::TBitmap();
	image->PixelFormat = pf24bit;
	image->Width = AGlyph->Width;
	image->Height = AGlyph->Height;
	image->Canvas->Draw(0, 0, AGlyph);
	const TColor transparentColor = GetPixel(image, 0, image->Height - 1);
	for(int y = image->Height - 1; y >= 0; y--) {
		for(int x = image->Width - 1; x >= 0; x--) {
			{
				const TColor srcPixel = GetPixel(image, x, y);
				if(srcPixel != transparentColor && srcPixel != clWhite) {
					SetPixel(image, x, y, FShadowColor);
					continue;
				}
			}
			if(x > 0 && y > 0) {
				const TColor srcPixel = GetPixel(image, x - 1, y - 1);
				if(srcPixel != transparentColor && srcPixel != clWhite) {
					SetPixel(image, x, y, FLightColor);
					continue;
				}
			}
			SetPixel(image, x, y, transparentColor);
		}
	}
	AGlyph->Width = AGlyph->Width * 2;
	AGlyph->Canvas->CopyRect(
		Bounds(AGlyph->Width / 2, 0, AGlyph->Width / 2, AGlyph->Height),
		image->Canvas,
		Bounds(0, 0, image->Width, image->Height));
}
//---------------------------------------------------------------------------
inline void __fastcall TGlyphConvertCommand::SaveGlyph(Graphics::TBitmap* AGlyph)
{
	AnsiString file = IncludeTrailingPathDelimiter(FOutputDir) + ExtractFileName(FFile);
	try {
		if(FileExists(file)) {
			throw EGlyphConvertError("Output file already exists: " + file);
		}
		AGlyph->SaveToFile(file);
	}
	catch(EStreamError& e) {
		throw EGlyphConvertError("Cannot save glyph to file: " + file);
	}
}
//---------------------------------------------------------------------------
inline TColor __fastcall TGlyphConvertCommand::GetPixel(Graphics::TBitmap* ABitmap, int AX, int AY) const
{
	const byte* elements = static_cast<byte*>(ABitmap->ScanLine[AY]) + 3 * AX;
	return static_cast<TColor>(RGB(elements[2], elements[1], elements[0]));
}
//---------------------------------------------------------------------------
void __fastcall TGlyphConvertCommand::SetPixel(Graphics::TBitmap* ABitmap, int AX, int AY, TColor APixel) const
{
	byte* elements = static_cast<byte*>(ABitmap->ScanLine[AY]) + 3 * AX;
	elements[0] = GetBValue(APixel);
	elements[1] = GetGValue(APixel);
	elements[2] = GetRValue(APixel);
}
//---------------------------------------------------------------------------
__fastcall TGlyphConvertCommand::TGlyphConvertCommand(const AnsiString& AFile, TColor ALightColor, TColor AShadowColor, const AnsiString& AOutputDir) :
	TObject(),
	FFile(AFile),
	FLightColor(ALightColor),
	FShadowColor(AShadowColor),
	FOutputDir(AOutputDir)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TGlyphConvertCommand::~TGlyphConvertCommand()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TGlyphConvertCommand::Execute()
{
	TPtr<Graphics::TBitmap> glyph = new Graphics::TBitmap();
	LoadGlyph(glyph);
	EditGlyph(glyph);
	SaveGlyph(glyph);
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::FullGlyph::GlyphConvertCommand
//---------------------------------------------------------------------------

