//---------------------------------------------------------------------------
// Shell.cpp
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <UtilCls.h>
#include "Shell.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moochos { namespace Moa { namespace Shell {
//---------------------------------------------------------------------------
void __fastcall TFileDrop::EnsureNotFinished()
{
	if(Finished) {
		throw EFileDropError("The file drop operation is already finished.");
	}
}
//---------------------------------------------------------------------------
HDROP __fastcall TFileDrop::GetHandle()
{
	EnsureNotFinished();
	return reinterpret_cast<HDROP>(FMessage.WParam);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TFileDrop::GetFile(int AIndex)
{
	EnsureNotFinished();
	if(AIndex < 0 || AIndex >= Count) {
		throw EFileDropError("The index for a dropped file is out of the range.");
	}
	UINT toCopySize = ::DragQueryFile(Handle, AIndex, NULL, 0); // Not including the terminator
	const UINT bufferSize = MAX_PATH;
	TCHAR buffer[bufferSize] = {0};
	UINT didCopySize = ::DragQueryFile(Handle, AIndex, buffer, bufferSize); // Not including the terminator
	if(didCopySize != toCopySize) {
		throw EFileDropError("The program cannot get the path of a dropped file.");
	}
	return AnsiString(buffer, didCopySize);
}
//---------------------------------------------------------------------------
int __fastcall TFileDrop::GetCount()
{
	EnsureNotFinished();
	return ::DragQueryFile(Handle, 0xFFFFFFFF, NULL, 0);
}
//---------------------------------------------------------------------------
TPoint __fastcall TFileDrop::GetPoint()
{
	EnsureNotFinished();
	TPoint point;
	::DragQueryPoint(Handle, &point);
	return point;
}
//---------------------------------------------------------------------------
__fastcall TFileDrop::TFileDrop(const TMessage& AMessage, bool AAutoFinish) :
	FMessage(),
	FAutoFinish(AAutoFinish),
	FFinished(false)
{
	if(AMessage.Msg != WM_DROPFILES) {
		throw EFileDropError("The message is not WM_DROPFILES.");
	}
	FMessage = AMessage;
}
//---------------------------------------------------------------------------
__fastcall TFileDrop::~TFileDrop()
{
	if(AutoFinish) {
		Finish();
	}
}
//---------------------------------------------------------------------------
void __fastcall TFileDrop::GetFiles(TStrings* AFiles)
{
	TPtr<TStrings> files = new TStringList();
	files->Capacity = Count;
	for(int i = 0; i < Count; i++) {
		files->Add(Files[i]);
	}
	AFiles->Assign(files);
}
//---------------------------------------------------------------------------
void __fastcall TFileDrop::Finish()
{
	if(Finished) {
		return;
	}
	::DragFinish(Handle);
	FFinished = true;
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Moa::Shell
//---------------------------------------------------------------------------

