//---------------------------------------------------------------------------
// Tasks.cpp
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include "Tasks.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace Moochos { namespace Moa { namespace Tasks {
//---------------------------------------------------------------------------
void __fastcall TTask::TSyncThread::Execute()
{
	FOnExecute(this);
}
//---------------------------------------------------------------------------
__fastcall TTask::TSyncThread::TSyncThread(TNotifyEvent AOnExecute) :
	TThread(true),
	FOnExecute(AOnExecute)
{
	FreeOnTerminate = true;
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoStart()
{
	if(OnStart != NULL) {
		Synchronize(DoStartInMainThread);
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoStartInMainThread()
{
	if(OnStart != NULL) {
		OnStart(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoEnd()
{
	if(OnEnd != NULL) {
		Synchronize(DoEndInMainThread);
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoEndInMainThread()
{
	if(OnEnd != NULL) {
		OnEnd(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoCancel()
{
	if(OnCancel != NULL) {
		Synchronize(DoCancelInMainThread);
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoCancelInMainThread()
{
	if(OnCancel != NULL) {
		OnCancel(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoProgress()
{
	if(OnProgress != NULL) {
		Synchronize(DoProgressInMainThread);
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoProgressInMainThread()
{
	if(OnProgress != NULL) {
		OnProgress(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoIdle()
{
	if(OnIdle != NULL) {
		Synchronize(DoIdleInMainThread);
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::DoIdleInMainThread()
{
	if(OnIdle != NULL) {
		OnIdle(this);
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::SetProgress(int AProgress)
{
	if(AProgress == FProgress) {
		return;
	}
	FProgress = AProgress;
	DoProgress();
}
//---------------------------------------------------------------------------
void __fastcall TTask::SyncThreadExecute(TObject* ASender)
{
	Execute();
	FSyncThread = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TTask::Execute()
{
	Start();
	Main();
	End();
}
//---------------------------------------------------------------------------
void __fastcall TTask::Start()
{
	FStarted = true;
	DoStart();
}
//---------------------------------------------------------------------------
void __fastcall TTask::End()
{
	FEnded = true;
	DoEnd();
}
//---------------------------------------------------------------------------
void __fastcall TTask::Idle()
{
	DoIdle();
}
//---------------------------------------------------------------------------
void __fastcall TTask::Synchronize(TThreadMethod AMethod)
{
	if(GetCurrentThreadId() == System::MainThreadID) {
		AMethod();
	}
	else {
		FSyncThread->Synchronize(AMethod);
	}
}
//---------------------------------------------------------------------------
int __fastcall TTask::MakeProgress(int ACurrent, int AAll)
{
	if(ACurrent == 0 || AAll == 0) {
		return 0;
	}
	return static_cast<__int64>(ACurrent) * 100 / AAll;
}
//---------------------------------------------------------------------------
__fastcall TTask::TTask(TTaskMode AMode) :
	TObject(),
	FMode(AMode),
	FProgress(-1),
	FStarted(false),
	FEnded(false),
	FCanceled(false),
	FAutoFree(false),
	FSyncThread(NULL),
	FOnStart(NULL),
	FOnEnd(NULL),
	FOnCancel(NULL),
	FOnProgress(NULL),
	FOnIdle(NULL)
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TTask::~TTask()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TTask::Perform()
{
	if(Started) {
		throw ETaskError("The task is tried to perform for multiple times.");
	}
	if(Mode == tmForeground) {
		Execute();
	}
	else if(Mode == tmBackground) {
		FSyncThread = new TSyncThread(&SyncThreadExecute);
		FSyncThread->Resume();
	}
}
//---------------------------------------------------------------------------
void __fastcall TTask::Cancel()
{
	if(!IsBusy() || Canceled) {
		return;
	}
	FCanceled = true;
	DoCancel();
}
//---------------------------------------------------------------------------
bool __fastcall TTask::IsBusy()
{
	return Started && !Ended;
}
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Moa::Tasks
//---------------------------------------------------------------------------

