//---------------------------------------------------------------------------
#ifndef Moochos_Moa_Tasks_H
#define Moochos_Moa_Tasks_H
//---------------------------------------------------------------------------
// Tasks.h
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <Classes.hpp>
//---------------------------------------------------------------------------
namespace Moochos { namespace Moa { namespace Tasks {
//---------------------------------------------------------------------------
class ETaskError : public Exception
{
public:
	__fastcall ETaskError(const AnsiString& AMessage) : Exception(AMessage) {}
};
//---------------------------------------------------------------------------
enum TTaskMode
{
	tmForeground,
	tmBackground,
};
//---------------------------------------------------------------------------
class TTask : public TObject
{
private:
	class TSyncThread : public TThread
	{
	private:
		TNotifyEvent FOnExecute;
	protected:
		virtual void __fastcall Execute();
	public:
		__fastcall TSyncThread(TNotifyEvent AOnExecute);
		using TThread::Synchronize;
	};
	TTaskMode FMode;
	int FProgress;
	bool FStarted;
	bool FEnded;
	bool FCanceled;
	bool FAutoFree;
	TSyncThread* FSyncThread;
	TNotifyEvent FOnStart;
	TNotifyEvent FOnEnd;
	TNotifyEvent FOnCancel;
	TNotifyEvent FOnProgress;
	TNotifyEvent FOnIdle;
	void __fastcall DoStart();
	void __fastcall DoStartInMainThread();
	void __fastcall DoEnd();
	void __fastcall DoEndInMainThread();
	void __fastcall DoCancel();
	void __fastcall DoCancelInMainThread();
	void __fastcall DoProgress();
	void __fastcall DoProgressInMainThread();
	void __fastcall DoIdle();
	void __fastcall DoIdleInMainThread();
	void __fastcall SetProgress(int AProgress);
	void __fastcall SyncThreadExecute(TObject* ASender);
protected:
	virtual void __fastcall Execute();
	virtual void __fastcall Start();
	virtual void __fastcall Main() = 0;
	virtual void __fastcall End();
	virtual void __fastcall Idle();
	virtual void __fastcall Synchronize(TThreadMethod AMethod);
public:
	static int __fastcall MakeProgress(int ACurrent, int AAll);
	__fastcall TTask(TTaskMode AMode);
	virtual __fastcall ~TTask();
	void __fastcall Perform();
	void __fastcall Cancel();
	bool __fastcall IsBusy();
	__property TTaskMode Mode = {read=FMode};
	__property int Progress = {read=FProgress, write=SetProgress};
	__property bool Started = {read=FStarted};
	__property bool Ended = {read=FEnded};
	__property bool Canceled = {read=FCanceled};
	__property bool AutoFree = {read=FAutoFree, write=FAutoFree};
	__property TNotifyEvent OnStart = {read=FOnStart, write=FOnStart};
	__property TNotifyEvent OnEnd = {read=FOnEnd, write=FOnEnd};
	__property TNotifyEvent OnCancel = {read=FOnCancel, write=FOnCancel};
	__property TNotifyEvent OnProgress = {read=FOnProgress, write=FOnProgress};
	__property TNotifyEvent OnIdle = {read=FOnIdle, write=FOnIdle};
};
//---------------------------------------------------------------------------
}}} // End of namespace Moochos::Moa::Tasks
using namespace Moochos::Moa::Tasks;
//---------------------------------------------------------------------------
#endif

