//---------------------------------------------------------------------------
// MainFormUnit.cpp
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <Moochos/Moa/Shell.h>
#include "MainFormUnit.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TMainForm* MainForm = NULL;
//---------------------------------------------------------------------------
namespace {
//---------------------------------------------------------------------------
const TColor DefLightColor = clWhite;
const TColor DefShadowColor = clGray;
const AnsiString DefOutputDir = IncludeTrailingPathDelimiter(GetCurrentDir()) + "Full Glyph Output";
//---------------------------------------------------------------------------
} // End of namespace
//---------------------------------------------------------------------------
__fastcall EInvalidFormInput::EInvalidFormInput(const AnsiString& AMessage, TWinControl* AControl) :
	Exception(AMessage)
{
	if(AControl != NULL && AControl->CanFocus()) {
		AControl->SetFocus();
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ApplicationIdle(TObject* ASender, bool& ADone)
{
	Application->OnIdle = NULL;
	ProcessParams();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCloseQuery(TObject* ASender, bool& ACanClose)
{
	ValidateInputs();
	SettingsMan->Save();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Edit1Exit(TObject* ASender)
{
	ValidateInputs();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ActionList1Update(TBasicAction* AAction, bool& AHandled)
{
	AHandled = true;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::GlyphsConvertFilesAcExecute(TObject* ASender)
{
	GlyphsMan->ChooseFiles(Files);
	GlyphsConvertAc->Execute();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::GlyphsConvertDroppedFilesAcExecute(TObject* ASender)
{
	GlyphsConvertAc->Execute();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::GlyphsConvertAcExecute(TObject* ASender)
{
	try {
		GlyphsMan->Convert(Files, LightColor, ShadowColor, OutputDir);
	}
	__finally {
		Files->Clear();
	}
	UpdateAccess();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::GlyphsSaveLogAsAcExecute(TObject* ASender)
{
	GlyphsMan->SaveLogAs();
	UpdateAccess();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::GlyphsDeleteLogAcExecute(TObject* ASender)
{
	GlyphsMan->DeleteLog(true);
	UpdateAccess();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::GlyphsExitAcExecute(TObject* ASender)
{
	Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::HelpContentsAcExecute(TObject* ASender)
{
	HelpMan->ShowContents(true);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::HelpAboutAcExecute(TObject* ASender)
{
	HelpMan->ShowAbout();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::InputLightColorAcExecute(TObject* ASender)
{
	LightColor = InputMan->ChooseColor(LightColor);
	UpdateAccess();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::InputShadowColorAcExecute(TObject* ASender)
{
	ShadowColor = InputMan->ChooseColor(ShadowColor);
	UpdateAccess();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::InputOutputDirAcExecute(TObject* ASender)
{
	OutputDir = InputMan->ChooseDirectory("Choose the output directory.", OutputDir);
	UpdateAccess();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Initialize()
{
	try {
		Application->OnIdle = ApplicationIdle;
		::DragAcceptFiles(Handle, true);
		FGlyphsMan = new TGlyphsMan();
		FHelpMan = new THelpMan();
		FInputMan = new TInputMan();
		FSettingsMan = new TSettingsMan();
		FFiles = new TStringList();
		LightColor = DefLightColor;
		ShadowColor = DefShadowColor;
		OutputDir = DefOutputDir;
		UpdateAccess();
		SettingsMan->Load();
		SelectFirst();
	}
	catch(...) {
		Uninitialize();
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Uninitialize()
{
	delete FFiles;
	FFiles = NULL;

	delete FSettingsMan;
	FSettingsMan = NULL;

	delete FInputMan;
	FInputMan = NULL;

	delete FHelpMan;
	FHelpMan = NULL;

	delete FGlyphsMan;
	FGlyphsMan = NULL;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ProcessParams()
{
	if(ParamCount() == 0) {
		return;
	}
	Application->MessageBox(
		"This program does not accept parameters.",
		Application->Title.c_str(),
		MB_OK | MB_ICONERROR | MB_SETFOREGROUND);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ValidateInputs()
{
	GetOutputDir();
}
//---------------------------------------------------------------------------
TColor __fastcall TMainForm::GetLightColor()
{
	return GetColorInput(Label2);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SetLightColor(TColor ALightColor)
{
	SetColorInput(Label2, Label3, ALightColor);
}
//---------------------------------------------------------------------------
TColor __fastcall TMainForm::GetShadowColor()
{
	return GetColorInput(Label5);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SetShadowColor(TColor AShadowColor)
{
	SetColorInput(Label5, Label6, AShadowColor);
}
//---------------------------------------------------------------------------
AnsiString __fastcall TMainForm::GetOutputDir()
{
	if(Edit1->Text.Length() == 0) {
		throw EInvalidFormInput("Specify the path of a directory.", Edit1);
	}
	return Edit1->Text;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SetOutputDir(AnsiString AOutputDir)
{
	Edit1->Text = AOutputDir;
}
//---------------------------------------------------------------------------
TColor __fastcall TMainForm::GetColorInput(TLabel* ASampleLabel)
{
	return ASampleLabel->Color;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::SetColorInput(TLabel* ASampleLabel, TLabel* AValueLabel, TColor AColor)
{
	ASampleLabel->Color = AColor;
	AValueLabel->Caption = AnsiString().sprintf(
		"[%u, %u, %u]",
		static_cast<unsigned int>(GetRValue(AColor)),
		static_cast<unsigned int>(GetGValue(AColor)),
		static_cast<unsigned int>(GetBValue(AColor)));
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::UpdateAccess()
{
	GlyphsSaveLogAsAc->Enabled = GlyphsMan->HasLog();
	GlyphsDeleteLogAc->Enabled = GlyphsMan->HasLog();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WMDropFiles(TMessage& AMessage)
{
	TFileDrop fileDrop(AMessage);
	::SetForegroundWindow(Handle);
	fileDrop.GetFiles(Files);
	fileDrop.Finish();
	GlyphsConvertAc->Execute();
}
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* AOwner) :
	TForm(AOwner),
	FGlyphsMan(NULL),
	FHelpMan(NULL),
	FInputMan(NULL),
	FSettingsMan(NULL),
	FFiles(NULL)
{
	Initialize();
}
//---------------------------------------------------------------------------
__fastcall TMainForm::~TMainForm()
{
	Uninitialize();
}
//---------------------------------------------------------------------------

