//---------------------------------------------------------------------------
// SettingsMan.cpp
//
// Version: 1.0.0
// Author: Moochos
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop
#include <UtilCls.h>
#include <Moochos/Moa/IniFiles.h>
#include "MainFormUnit.h"
#include "SettingsMan.h"
//---------------------------------------------------------------------------
namespace Moochos { namespace FullGlyph { namespace SettingsMan {
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
namespace {
//---------------------------------------------------------------------------
const AnsiString IniFile = ExtractFilePath(Application->ExeName) + "Settings.ini";
const AnsiString LoadErrorMessage = "The program cannot load the settings.";
const AnsiString SaveErrorMessage = "The program cannot save the settings.";
//---------------------------------------------------------------------------
} // End of namespace
//---------------------------------------------------------------------------
void __fastcall TSettingsMan::LoadMainFormSection(TVarIniFile* AIni)
{
	AnsiString section = "MainForm";
	MainForm->Left = AIni->ReadInteger(section, "Left", MainForm->Left);
	MainForm->Top = AIni->ReadInteger(section, "Top", MainForm->Top);
	MainForm->GlyphsMan->FilesOpenDialog->FilterIndex = AIni->ReadInteger(section, "GlyphsMan.FilesOpenDialog.FilterIndex", MainForm->GlyphsMan->FilesOpenDialog->FilterIndex);
	MainForm->GlyphsMan->LogSaveDialog->FilterIndex = AIni->ReadInteger(section, "GlyphsMan.LogSaveDialog.FilterIndex", MainForm->GlyphsMan->LogSaveDialog->FilterIndex);
	MainForm->LightColor = static_cast<TColor>(AIni->ReadInteger(section, "LightColor", MainForm->LightColor));
	MainForm->ShadowColor = static_cast<TColor>(AIni->ReadInteger(section, "ShadowColor", MainForm->ShadowColor));
	MainForm->OutputDir = AIni->ReadQuotedString(section, "OutputDir", MainForm->OutputDir);
}
//---------------------------------------------------------------------------
void __fastcall TSettingsMan::SaveMainFormSection(TVarIniFile* AIni)
{
	AnsiString section = "MainForm";
	AIni->WriteInteger(section, "Left", MainForm->Left);
	AIni->WriteInteger(section, "Top", MainForm->Top);
	AIni->WriteInteger(section, "GlyphsMan.FilesOpenDialog.FilterIndex", MainForm->GlyphsMan->FilesOpenDialog->FilterIndex);
	AIni->WriteInteger(section, "GlyphsMan.LogSaveDialog.FilterIndex", MainForm->GlyphsMan->LogSaveDialog->FilterIndex);
	AIni->WriteInteger(section, "LightColor", MainForm->LightColor);
	AIni->WriteInteger(section, "ShadowColor", MainForm->ShadowColor);
	AIni->WriteQuotedString(section, "OutputDir", MainForm->OutputDir);
}
//---------------------------------------------------------------------------
void __fastcall TSettingsMan::LoadSystemSection(TVarIniFile* AIni)
{
	AnsiString section = "System";
	SetCurrentDir(AIni->ReadQuotedString(section, "CurrentDir", GetCurrentDir()));
}
//---------------------------------------------------------------------------
void __fastcall TSettingsMan::SaveSystemSection(TVarIniFile* AIni)
{
	AnsiString section = "System";
	AIni->WriteQuotedString(section, "CurrentDir", GetCurrentDir());
}
//---------------------------------------------------------------------------
__fastcall TSettingsMan::TSettingsMan() :
	TObject()
{
	// Do nothing
}
//---------------------------------------------------------------------------
__fastcall TSettingsMan::~TSettingsMan()
{
	// Do nothing
}
//---------------------------------------------------------------------------
void __fastcall TSettingsMan::Load()
{
	if(!FileExists(IniFile)) {
		return;
	}
	try {
		TPtr<TVarIniFile> ini = new TVarIniFile(IniFile);
		LoadMainFormSection(ini);
		LoadSystemSection(ini);
	}
	catch(EIniFileException& e) {
		throw ESettingsManError(LoadErrorMessage);
	}
}
//---------------------------------------------------------------------------
void __fastcall TSettingsMan::Save()
{
	try {
		TPtr<TVarIniFile> ini = new TVarIniFile(IniFile);
		ini->Clear();
		SaveMainFormSection(ini);
		SaveSystemSection(ini);
		ini->UpdateFile();
	}
	catch(EIniFileException& e) {
		throw ESettingsManError(SaveErrorMessage);
	}
}
//---------------------------------------------------------------------------
}}} // End of Moochos::FullGlyph::SettingsMan
//---------------------------------------------------------------------------

